#include "Destination.h"

Destination::Destination(char *host, int port,
			 nexus_global_pointer_t *cb_gp,
			 int n_packet_types,
			 int *packet_types) :
			 callback_gp_(cb_gp),
			 rtp_address_(port, host),
			 rtcp_address_(port + 1, host),
			 rtp_sock_(ACE_Addr::sap_any),
			 rtcp_sock_(ACE_Addr::sap_any)
{
    int i;
    
    printf("Created destination %x for %s/%d\n", this,
	   rtp_address_.get_host_name(), rtp_address_.get_port_number());

    for (i = 0; i < MAX_PACKET_TYPES; i++)
    {
	want_packet_[i] = 0;
    }
    
    printf("Types wanted:");
    for (i = 0; i < n_packet_types; i++)
    {
	printf(" %d", packet_types[i]);
	want_packet_[packet_types[i]] = 1;
    }
    printf("\n");
}

int Destination::matches(char *host, int port)
{
    return (strcmp(host, rtp_address_.get_host_name()) == 0 &&
	    ((port == 0) ||
	     (port == (int) rtp_address_.get_port_number())));
}

int Destination::wants_type(int packet_type)
{
    return want_packet_[packet_type];
}

void Destination::send_rtp_packet(int rtp_type, char *packet, int len)
{
    int i;

    if (want_packet_[rtp_type])
    {
	if (rtp_sock_.send((const void *) packet, len, rtp_address_) < 0)
	{
	    perror("send failed");
	}
    }
}

void Destination::send_rtcp_packet(char *packet, int len)
{
    int i;

    if (rtcp_sock_.send((const void *) packet, len, rtcp_address_) < 0)
    {
	perror("send failed");
    }
}

void Destination::send_rtcp_packet(int rtp_type, char *packet, int len)
{
    int i;

    if (want_packet_[rtp_type])
    {
	if (rtcp_sock_.send((const void *) packet, len, rtcp_address_) < 0)
	{
	    perror("send failed");
	}
    }
}

void Destination::send_callback(char *handler, char *msg)
{
    nexus_buffer_t buf;
    int len;
    int size, n_elts, e;
    
    nexus_init_remote_service_request(&buf, callback_gp_, handler,
				      nexus_handler_hash(handler));

    len = strlen(msg);
    
    size = n_elts = 0;
    size += nexus_sizeof_int(&buf, 1);
    n_elts += 1;
    
    size += nexus_sizeof_char(&buf, len);
    n_elts += 1;
    
    nexus_set_buffer_size(&buf, size, n_elts);
    nexus_put_int(&buf, &len, 1);
    nexus_put_char(&buf, msg, len);
//    printf("Sending callback %s\n", msg);
    nexus_send_remote_service_request(&buf);
}
