#ifndef _Disk_Writer_h
#define _Disk_Writer_h

#include "RTP_Packet_Sink.h"
#include "RTCP_Packet_Sink.h"
#include "SSRC_Writer.h"
#include "ace/Event_Handler.h"
#include "SSRC_Hash.h"

//class Reflector;

#if defined (ACE_HAS_SIGINFO_T)
#define HANDLE_SIGNAL_ARGS , siginfo_t * = 0, ucontext_t * = 0
#else
#define HANDLE_SIGNAL_ARGS
#endif /* ACE_HAS_SIGINFO_T */

/*
 * Derive from packet sinks so we can grab packets.
 *
 * Derive from ACE_Event_Handler so we can grab timer ticks (for
 * RTCP saves) and signals.
 *
 * The Disk_Writer writes the individual data streams into separate
 * files, distinguished by ssrc. For each ssrc, we maintain a record
 * of the formats used in that stream (we don't just mark the format
 * for a stream, as the format can change).
 *
 * For now, we write files in the rtpdump format for easy playback.
 *
 * The file header is
 *
 *	struct timeval start time
 *	unsigned long source (ip address of source)
 *	unsigned short port (port of source)     
 *
 * Each packet is headed by
 *
 *	unsigned short length
 *	unsigned short type (rtp or rtcp)
 *	unsigned long offset (ms since start of recording)
 *	
 * and followed by the original rtp or rtcp packet data.
 *
 * The logic for the writing of ssrc data is encapsulated in the
 * SSRC_Writer object.
 *
 * Packets are vectored to the appropriate writer via the
 * DW's SSRC_KeySet (out of the IBM class library)
 * 
 */

class Disk_Writer : public RTP_Packet_Sink, public RTCP_Packet_Sink,
		    public ACE_Event_Handler
{
public:
    Disk_Writer(char *directory);
    ~Disk_Writer();

    int finished() { return finished_; } ;
    void terminate();

    const char *directory() { return directory_; } ;

    void ssrc_finished(SSRC_Writer *writer);

//    void setup_reflector(char *host, unsigned short *ports, int n_ports);
    
protected:
    virtual void handle_packet(RTP_Packet::RTP_Header *packet_header,
			       char *packet, int packet_size,
			       ACE_INET_Addr &packet_source);
    
    virtual void handle_packet(RTCP_Packet::RTCP_Header *packet_header,
			       char *packet, int packet_size,
			       ACE_INET_Addr &packet_source);

    virtual void handle_subpacket(RTCP_Packet::RTCP_Header *packet_header,
				  char *packet, int packet_size,
				  ACE_INET_Addr &packet_source);

    virtual int handle_timeout(const ACE_Time_Value &tv, const void *arg);
    virtual int handle_signal (ACE_HANDLE signum HANDLE_SIGNAL_ARGS);

    char *directory_;
//    SSRC_Writer_KeySet writer_keyset_;
    SSRC_Hash writer_hash;

    int finished_;
    ACE_Time_Value last_tick_;

//    Reflector *reflector;
};

#endif /* _Disk_Writer_h */
