#ifndef _Media_Stream_h
#define _Media_Stream_h

/*
 * A Media_Stream is the encapsulation of the RTP & RTCP sockets
 * for a stream.
 *
 */

#include "ace/SOCK_Dgram.h"
#include "ace/INET_Addr.h"
#include "ace/Reactor.h"

#include "RTP_Packet.h"
#include "RTCP_Packet.h"
#include "Packet_Handler.h"
#include "RTP_Packet_Handler.h"
#include "RTCP_Packet_Handler.h"
#include "RTP_Packet_Sink.h"
#include "RTCP_Packet_Sink.h"

class Media_Stream
{
public:
    Media_Stream(unsigned short port,
		 RTP_Packet_Sink *rtp_sink,
		 RTCP_Packet_Sink *rtcp_sink);
    ~Media_Stream(void);

    void register_with_reactor(ACE_Reactor *reactor);

    unsigned short port(void) 	{ return port_; };

    ACE_INET_Addr & rtp_addr() 	{ return rtp_addr_; };
    ACE_INET_Addr & rtcp_addr() 	{ return rtcp_addr_; };
    ACE_SOCK_Dgram & rtp_socket() 	{ return rtp_socket_; };
    ACE_SOCK_Dgram & rtcp_socket()	{ return rtcp_socket_; };

    RTP_Packet_Handler & rtp_handler()		{ return rtp_handler_; };
    RTCP_Packet_Handler & rtcp_handler()	{ return rtcp_handler_; };

    RTP_Packet_Sink *rtp_sink()		{ return rtp_sink_; };
    RTCP_Packet_Sink *rtcp_sink()	{ return rtcp_sink_; };

protected:

    unsigned short port_;

    ACE_INET_Addr rtp_addr_;
    ACE_INET_Addr rtcp_addr_;

    ACE_SOCK_Dgram rtp_socket_;
    ACE_SOCK_Dgram rtcp_socket_;

    RTP_Packet_Handler rtp_handler_;
    RTCP_Packet_Handler rtcp_handler_;

    RTP_Packet_Sink *rtp_sink_;
    RTCP_Packet_Sink *rtcp_sink_;
};

#endif /* _Media_Stream_h */
