#
# Representation of remote pwriter
#

package PWriter;

sub new
{
    my($class, $gp) = @_;

    my($self) = {
	gp => $gp,
    };

    bless($self, $class);

    return $self;
}

sub call_rsr_string
{
    my($self, $rsr, $string) = @_;
    my($buf);
    my($size, $n_elts, $s, $e);

    $buf = Nexus::init_rsr($self->{gp}, $rsr);
    $size = $n_elts = 0;
    ($s, $e) = $buf->sizeof_int(1);
    $size += $s; $n_elts += $e;
    
    ($s, $e) = $buf->sizeof_char(length($string));
    $size += $s; $n_elts += $e;
    
    $buf->set_size($size, $n_elts);
    $buf->put_int(length($string));
    $buf->put_string($string);
    $buf->send_rsr();
}

sub call_rsr_string_intarray
{
    my($self, $rsr, $string, @inta) = @_;
    my($buf);
    my($size, $n_elts, $s, $e, $nint);

    $buf = Nexus::init_rsr($self->{gp}, $rsr);
    $size = $n_elts = 0;
    ($s, $e) = $buf->sizeof_int(1);
    $size += $s; $n_elts += $e;
    
    ($s, $e) = $buf->sizeof_char(length($string));
    $size += $s; $n_elts += $e;

    $nint = @inta;
    ($s, $e) = $buf->sizeof_int($nint + 1);
    $size += $s; $n_elts += $e;
    
    $buf->set_size($size, $n_elts);
    $buf->put_int(length($string));
    $buf->put_string($string);

    $buf->put_int($nint);
    for $e (@inta)
    {
	$buf->put_int($e);
    }

    $buf->send_rsr();
}

sub call_rsr_string_int
{
    my($self, $rsr, $string, $int) = @_;
    my($buf);
    my($size, $n_elts, $s, $e);

    $buf = Nexus::init_rsr($self->{gp}, $rsr);
    $size = $n_elts = 0;
    ($s, $e) = $buf->sizeof_int(1);
    $size += $s; $n_elts += $e;
    
    ($s, $e) = $buf->sizeof_char(length($string));
    $size += $s; $n_elts += $e;
    
    ($s, $e) = $buf->sizeof_int(1);
    $size += $s; $n_elts += $e;
    
    $buf->set_size($size, $n_elts);
    $buf->put_int(length($string));
    $buf->put_string($string);
    $buf->put_int($int);
    $buf->send_rsr();
}

sub call_rsr_string_int_intarray
{
    my($self, $rsr, $string, $int, @inta) = @_;
    my($buf);
    my($size, $n_elts, $s, $e, $nint);

    $buf = Nexus::init_rsr($self->{gp}, $rsr);
    $size = $n_elts = 0;
    ($s, $e) = $buf->sizeof_int(1);
    $size += $s; $n_elts += $e;
    
    ($s, $e) = $buf->sizeof_char(length($string));
    $size += $s; $n_elts += $e;
    
    ($s, $e) = $buf->sizeof_int(1);
    $size += $s; $n_elts += $e;

    $nint = @inta;
    ($s, $e) = $buf->sizeof_int($nint + 1);
    $size += $s; $n_elts += $e;
    
    $buf->set_size($size, $n_elts);
    $buf->put_int(length($string));
    $buf->put_string($string);
    $buf->put_int($int);
    $buf->put_int($nint);
    for $e (@inta)
    {
	$buf->put_int($e);
    }
    
    $buf->send_rsr();
}

sub call_rsr_string_int_gp_string_intarray
{
    my($self, $rsr, $string, $int, $gp, $string2, @inta) = @_;
    my($buf);
    my($size, $n_elts, $s, $e, $nint);

    print "call_rsr_string_int_gp_string_intarray $rsr $string $int $gp $string2 @inta\n";

    $buf = Nexus::init_rsr($self->{gp}, $rsr);
    $size = $n_elts = 0;

    #
    # String
    #
    ($s, $e) = $buf->sizeof_int(1);
    $size += $s; $n_elts += $e;
    
    ($s, $e) = $buf->sizeof_char(length($string));
    $size += $s; $n_elts += $e;

    #
    # Int
    #
    
    ($s, $e) = $buf->sizeof_int(1);
    $size += $s; $n_elts += $e;

    #
    # Gp
    #
    
    ($s, $e) = $buf->sizeof_global_pointer($gp);
    $size += $s; $n_elts += $e;

    #
    # String
    #
    ($s, $e) = $buf->sizeof_int(1);
    $size += $s; $n_elts += $e;
    
    ($s, $e) = $buf->sizeof_char(length($string2));
    $size += $s; $n_elts += $e;

    #
    # Intarray
    #

    $nint = @inta;
    ($s, $e) = $buf->sizeof_int($nint + 1);
    $size += $s; $n_elts += $e;
    
    $buf->set_size($size, $n_elts);

    $buf->put_int(length($string));
    $buf->put_string($string);

    $buf->put_int($int);

    $buf->put_global_pointer($gp);
    
    $buf->put_int(length($string2));
    $buf->put_string($string2);

    $buf->put_int($nint);
    for $e (@inta)
    {
	$buf->put_int($e);
    }
    
    $buf->send_rsr();
}

sub call_rsr_string_int_gp_intarray
{
    my($self, $rsr, $string, $int, $gp, @inta) = @_;
    my($buf);
    my($size, $n_elts, $s, $e, $nint);

    $buf = Nexus::init_rsr($self->{gp}, $rsr);
    $size = $n_elts = 0;

    #
    # String
    #
    ($s, $e) = $buf->sizeof_int(1);
    $size += $s; $n_elts += $e;
    
    ($s, $e) = $buf->sizeof_char(length($string));
    $size += $s; $n_elts += $e;

    #
    # Int
    #
    
    ($s, $e) = $buf->sizeof_int(1);
    $size += $s; $n_elts += $e;

    #
    # Gp
    #
    
    ($s, $e) = $buf->sizeof_global_pointer($gp);
    $size += $s; $n_elts += $e;

    #
    # Intarray
    #

    $nint = @inta;
    ($s, $e) = $buf->sizeof_int($nint + 1);
    $size += $s; $n_elts += $e;
    
    $buf->set_size($size, $n_elts);

    $buf->put_int(length($string));
    $buf->put_string($string);

    $buf->put_int($int);

    $buf->put_global_pointer($gp);
    
    $buf->put_int($nint);
    for $e (@inta)
    {
	$buf->put_int($e);
    }
    
    $buf->send_rsr();
}

sub call_rsr_gp
{
    my($self, $rsr, $gp) = @_;
    my($buf);
    my($size, $n_elts, $s, $e);

    $buf = Nexus::init_rsr($self->{gp}, $rsr);
    $size = $n_elts = 0;
    ($s, $e) = $buf->sizeof_global_pointer($gp);
    $size += $s; $n_elts += $e;
    
    $buf->set_size($size, $n_elts);
    $buf->put_global_pointer($gp);
    $buf->send_rsr();
}

sub call_rsr_void
{
    my($self, $rsr) = @_;
    my($buf);
    my($size, $n_elts, $s, $e);

    $buf = Nexus::init_rsr($self->{gp}, $rsr);
    $size = $n_elts = 0;
    $buf->set_size($size, $n_elts);
    $buf->send_rsr();
}

sub add_client
{
    my($self, $client_gp) = @_;

    $self->call_rsr_gp("add_client", $client_gp);
}

sub add_file
{
    my($self, $path, @packet_types) = @_;

    $self->call_rsr_string_intarray("add_file", $path, @packet_types);
}

sub remove_file
{
    my($self, $path) = @_;

    $self->call_rsr_string("remove_file", $path);
}

sub add_destination
{
    my($self, $host, $port, $cb_gp, @packet_types) = @_;

    $self->call_rsr_string_int_gp_intarray("add_destination", $host,
						  $port,
						  $cb_gp, 
						  @packet_types);
}

sub remove_destination
{
    my($self, $host, $port) = @_;

    $self->call_rsr_string_int("remove_destination", $host, $port);
}

sub start
{
    my($self) = @_;

    $self->call_rsr_void("start");
}

sub stop
{
    my($self) = @_;

    $self->call_rsr_void("stop");
}

sub reset
{
    my($self) = @_;

    $self->call_rsr_void("reset");
}

sub quit
{
    my($self) = @_;

    $self->call_rsr_void("quit");
}

1;
