#ifndef _Packet_Handler_h
#define _Packet_Handler_h

/*
 * Packet handler. Awakened via the handle_input() method by the
 * ACE_Reactor when input ready.
 *
 * handle_input() will read the data into the Packet object
 * in the packet_ instance variable, and invoke the handle_packet()
 * method.
 *
 * The thought behind this is that we derive RTP_Packet and RTCP_Packet
 * from Packet, derive RTP_Packet_Handler and RTCP_Packet_Handler from
 * Packet_Handler, and everything Just Works. The interconnections needed
 * will be that the RTP & RTCP packet handlers know about each other; they
 * will also have to know about the ACE_SOCK_Dgram objects that they
 * are associated with. 
 *
 * (This information could get bundled into a Media_Stream object that
 * keeps track of higher-level information as well. Then the packet handlers
 * need only keep a pointer to the Media_Stream)
 *
 * (Then a Media_Stream can be derived into a RTP_Media_Stream, thence
 * to a JPEG_Media_Stream, and so on. The jpeg frame reassembler
 * would live in the stream)
 *
 * But for the first cut, we'll make it simple. Just one type of
 * Packet_Handler, just one type of Packet. Specific methods within
 * Packet_Handler to handle the different types we see (there are
 * twitchinesses with this anyway, since one doesn't know which of the
 * different RTP/RTCP packet subtypes one has without inspecting
 * the packet). It sure would be nice, tho, to be able to access
 * the packet flags without having to do typecasting. We can do this
 * by making Packet a general purpose thang, with a union of the
 * rtp and rtcp headers in there, or somesuch. 
 */

#include "ace/Event_Handler.h"
#include "ace/SOCK_Dgram.h"
#include "ace/INET_Addr.h"
#include "Packet.h"

class Packet_Handler : public ACE_Event_Handler
{
public:
    
    Packet_Handler(ACE_SOCK_Dgram *sock = NULL);
    ~Packet_Handler(void);

    void set_socket(ACE_SOCK_Dgram *sock) { socket_ = sock; };
    ACE_SOCK_Dgram *get_socket() { return socket_; };
    
    virtual int handle_input(ACE_HANDLE fd);
    virtual ACE_HANDLE get_handle(void) const;

    virtual int handle_packet(char *buffer, int size, ACE_INET_Addr &from) = 0;

    virtual char *get_buffer(int *size) = 0;

protected:

    ACE_SOCK_Dgram *socket_;
};

#endif /*  _Packet_Handler_h */
