#ifndef _RTCP_Packet_h
#define _RTCP_Packet_h

/*
 * This brings in ntohs() and friends.
 */
#include "ace/SOCK.h"

#include "defs.h"

#include "Packet.h"

/*

 0                   1                   2                   3
 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|V=2|P|    RC   |   PT=SR=200   |             length            | header
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|                         SSRC of sender                        |

*/


class RTCP_Packet : public Packet
{
public:
    
    struct RTCP_Header
    {
	unsigned short flags_;
	unsigned short length_;
	unsigned long ssrc_;

	unsigned short get_flags() { return ntohs(flags_); };
	
	unsigned short get_version() {return ((get_flags() & 0xc000) >> 14); };
	unsigned short get_padding() {return ((get_flags() & 0x2000) >> 13); };
	unsigned short get_reception_count() {return ((get_flags() & 0x1f00) >> 8);};
	unsigned short get_packet_type() { return  ((get_flags() & 0xff)); };

	unsigned short get_length() { return ntohs(length_); };
	unsigned long get_ssrc() { return ntohl(ssrc_); };
    };

    struct Sender_Info
    {
	unsigned long ntp_timestamp_msw_;
	unsigned long ntp_timestamp_lsw_;
	unsigned long rtp_timestamp_;
	unsigned long packet_count_;
	unsigned long octet_count_;

	unsigned long get_ntp_timestamp_msw() { return ntohl(ntp_timestamp_msw_); } ;
	unsigned long get_ntp_timestamp_lsw() { return ntohl(ntp_timestamp_lsw_); } ;
	unsigned long get_rtp_timestamp() { return ntohl(rtp_timestamp_); } ;
	unsigned long get_octet_count() { return ntohl(octet_count_); } ;
	unsigned long get_packet_count() { return ntohl(packet_count_); } ;


	char *as_string();
    };

    struct Report_Block
    {
	unsigned long ssrc_;
	unsigned long loss_info_;
	unsigned long ext_highest_seq_;
	unsigned long jitter_;
	unsigned long last_sr_;
	unsigned long delay_since_last_sr_;

	unsigned long get_ssrc() { return ntohl(ssrc_); } ;
	unsigned long get_loss_info() { return ntohl(loss_info_); } ;
	unsigned long get_ext_highest_seq() { return ntohl(ext_highest_seq_); } ;
	unsigned long get_jitter() { return ntohl(jitter_); } ;
	unsigned long get_last_sr() { return ntohl(last_sr_); } ;
	unsigned long get_delay_since_last_sr() { return ntohl(delay_since_last_sr_); } ;

	char *as_string();
    };

    struct SDES
    {
	unsigned char type;
	unsigned char length;
	unsigned char data[257];
    };

    enum Payload_Types
    {
	PT_SR = 200,
	PT_RR = 201,
	PT_SDES = 202,
	PT_BYE = 203,
	PT_APP = 204,

	PT_SR_OLD = 0,
	PT_RR_OLD = 1,
	PT_SDES_OLD = 2,
	PT_BYE_OLD = 3,
	PT_APP_OLD = 4
    };

    union
    {
	struct
	{
	    RTCP_Header header_;
	    char data[1];
	} pkt_header_;
	char buffer_[MAX_PACKET];
    } u;

    virtual char *get_buffer(int *size) { *size = sizeof(u.buffer_); return u.buffer_; };


};

#endif /* _RTCP_Packet_h */
