
#include "RTCP_Packet_Handler.h"

RTCP_Packet_Handler::RTCP_Packet_Handler(ACE_SOCK_Dgram *sock,
					 RTCP_Packet_Sink *sink) :
		Packet_Handler(sock),
		packet_sink_(sink)

{
}

RTCP_Packet_Handler::~RTCP_Packet_Handler(void)
{
}

int RTCP_Packet_Handler::handle_packet(char *buffer, int size,
				      ACE_INET_Addr &from)
{
    char *p;
    RTCP_Packet::RTCP_Header *hdr;
    

    p = buffer;

    hdr = (RTCP_Packet::RTCP_Header *) p;
    packet_sink_->handle_packet(hdr, buffer, size, from);

    while (p < (buffer + size))
    {
	hdr = (RTCP_Packet::RTCP_Header *) p;

	if (packet_sink_)
	{
	    packet_sink_->handle_subpacket(hdr,
					p,
					4 * (hdr->get_length() + 1),
					from);
	}

#if 0
	printf("Got rtcp packet of length %d\n", size);
	printf("flags=%x version=%d padding=%d rc=%d pt=%d len=%d ssrc=%x\n",
	       hdr->get_flags(),
	       hdr->get_version(),
	       hdr->get_padding(),
	       hdr->get_reception_count(),
	       hdr->get_packet_type(),
	       hdr->get_length(),
	       hdr->get_ssrc());


	switch (hdr->get_packet_type())
	{
	case RTCP_Packet::PT_SR:
	case RTCP_Packet::PT_SR_OLD:

	    handle_sr(hdr,
		      (RTCP_Packet::Sender_Info *) (p + sizeof(*hdr)),
		      hdr->get_reception_count(),
		      (RTCP_Packet::Report_Block *)
		      (p + sizeof(*hdr) + sizeof(RTCP_Packet::Sender_Info)));
	    break;

	case RTCP_Packet::PT_RR:
	case RTCP_Packet::PT_RR_OLD:

	    handle_rr(hdr,
		      hdr->get_reception_count(),
		      (RTCP_Packet::Report_Block *) (p + sizeof(*hdr)));
	    break;

	case RTCP_Packet::PT_SDES:
	case RTCP_Packet::PT_SDES_OLD:
	    {
		char *pp;
		int i, n_entries;
		RTCP_Packet::SDES *entries;
		n_entries = hdr->get_reception_count();

		entries = new RTCP_Packet::SDES[n_entries];

		for (pp = p + sizeof(*hdr), i = 0; i < n_entries; i++)
		{
		    entries[i].type = pp[0];
		    entries[i].length = pp[1];
		    strncpy((char *) entries[i].data, pp + 2, pp[1]);
		    entries[i].data[pp[1]] = '\0';
		    pp += pp[1] + 2;
		}
		handle_sdes(hdr, n_entries, entries);
		delete entries;
	    }
	    break;

	case RTCP_Packet::PT_BYE:
	case RTCP_Packet::PT_BYE_OLD:
	    {
		int n_entries = hdr->get_reception_count();
		unsigned long *entries, *raw;
		int i;

		entries = new unsigned long[n_entries];

		raw = (unsigned long *) (p + 4);
		for (i = 0; i < n_entries; i++)
		{
		    entries[i] = ntohl(raw[i]);
		}
		handle_bye(hdr, n_entries, entries);
		delete entries;
	    }
	    break;
	}
#endif
	    
	p += 4 * (hdr->get_length() + 1);
    }
    return 0;
}

#if 0
void RTCP_Packet_Handler::handle_sr(RTCP_Packet::RTCP_Header *header,
				    RTCP_Packet::Sender_Info *sender_info,
				    int n_report_blocks,
				    RTCP_Packet::Report_Block *report_blocks)
{
    int i;
    
    printf("handle_sr: n_rpt=%d sender_info=%s\n",
	   n_report_blocks, sender_info->as_string());

    for (i = 0; i < n_report_blocks; i++)
    {
	printf("  Rpt %d: %s\n", i, report_blocks[i].as_string());
    }
}

void RTCP_Packet_Handler::handle_rr(RTCP_Packet::RTCP_Header *header,
				    int n_report_blocks,
				    RTCP_Packet::Report_Block *report_blocks)
{
    int i;
    
    printf("handle_rr: n_rpt=%d\n", n_report_blocks);

    for (i = 0; i < n_report_blocks; i++)
    {
	printf("  Rpt %d: %s\n", i, report_blocks[i].as_string());
    }
}

void RTCP_Packet_Handler::handle_sdes(RTCP_Packet::RTCP_Header *header,
				      int n_entries,
				      RTCP_Packet::SDES *entries)
{
    int i;

    printf("handle_sdes: n=%d\n", n_entries);
    for (i = 0; i < n_entries; i++)
    {
	printf("  sdes %d: type=%d data=\"%s\"\n", i,
	       entries[i].type, entries[i].data);
    }
}

void RTCP_Packet_Handler::handle_bye(RTCP_Packet::RTCP_Header *header,
				      int n_entries,
				      unsigned long *entries)
{
    int i;

    printf("handle_bye: n=%d ", n_entries);
    for (i = 0; i < n_entries; i++)
    {
	printf("  %x", entries[i]);
    }
    printf("\n");
}
#endif

char *RTCP_Packet_Handler::get_buffer(int *size)
{
    return packet_buffer_.get_buffer(size);
}
