#ifndef _RTP_Packet_h
#define _RTP_Packet_h

/*
 * This brings in ntohs() and friends.
 */
#include "ace/SOCK.h"

#include "defs.h"

#include "Packet.h"

/*

 0                   1                   2                   3
 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|V=2|P|X|  CC   |M|     PT      |       sequence number         |
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|                           timestamp                           |
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|           synchronization source (SSRC) identifier            |
+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+
|            contributing source (CSRC) identifiers             |
|                             ....                              |
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+

*/

class RTP_Packet : public Packet
{
public:
    struct RTP_Header
    {
	unsigned short 	flags_;
	unsigned short	sequence_;
	unsigned long	timestamp_;
	unsigned long 	ssrc_;

	unsigned short get_flags() { return ntohs(flags_); };
	
	unsigned short get_version() {return ((get_flags() & 0xc000) >> 14); };
	unsigned short get_padding() {return ((get_flags() & 0x2000) >> 13); };
	unsigned short get_extension() {return ((get_flags()&0x1000) >> 12); };
	unsigned short get_csrc_count() {return ((get_flags() &0x0f00) >> 8);};
	unsigned short get_marker() { return  ((get_flags() &0x0080) >> 7);};
	unsigned short get_packet_type() { return  ((get_flags() & 0x7f)); };

	unsigned short get_sequence() { return ntohs(sequence_); };
	unsigned long get_timestamp() { return ntohl(timestamp_); };
	unsigned long get_ssrc() { return ntohl(ssrc_); };
    };

    struct JPEG_Header
    {
	unsigned long w1_;
	unsigned long w2_;

	unsigned long w1() { return ntohl(w1_); };
	unsigned long w2() { return ntohl(w2_); };
	
	unsigned char mbz() { return ((w1() & 0xff000000) >> 24); };
	unsigned long fragment_offset() { return (w1() & 0x00ffffff); };
	unsigned char type() { return ((w2() & 0xff000000) >> 24); };
	unsigned char q() { return ((w2() & 0x00ff0000) >> 16); };
	unsigned char width() { return ((w2() & 0x0000ff00) >> 8); };
	unsigned char height() { return (w2() & 0x000000ff); };
    };

    enum Payload_Types
    {
	PT_PCMU = 0,
	PT_G721 = 2,
	PT_GSM  = 3,
	PT_G723 = 4,
	PT_IDVI_8000 = 5,
	PT_IDVI_16000 = 6,
	PT_LPC = 7,
	PT_L16_STEREO = 10,
	PT_L16_MONO = 11,
	PT_SCR = 23,
	PT_MPEG = 24,
	PT_CELLB = 25,
	PT_JPEG = 26,
	PT_CUSEEME = 27,
	PT_NV = 28,
	PT_DVC = 29,
	PT_BOLT = 30,
	PT_H261 = 31
    };

    union
    {
	RTP_Header header_;
	char buffer_[MAX_PACKET];
    } u;

    virtual char *get_buffer(int *size) { *size = sizeof(u.buffer_); return u.buffer_; };
};

#endif /* _RTP_Packet_h */
