#include "Reflector.h"
#include <ace/Reactor.h>

Reflector::Reflector(unsigned short port, ACE_Reactor *reactor) :
			port_(port),
			reactor_(reactor),
			rtp_addr_(port_),
			rtcp_addr_(port_ + 1),
			rtp_socket_(rtp_addr_),
			rtcp_socket_(rtcp_addr_),
			rtp_handler_(&rtp_socket_, this),
			rtcp_handler_(&rtcp_socket_, this),
			send_socket_(ACE_Addr::sap_any)
{
    printf("Reflector::Reflector() port=%d\n", port_);

    if (reactor->register_handler(&rtp_handler_, ACE_Event_Handler::READ_MASK) < 0)
    {
	printf("Register_handler for rtp_handler for port %d failed\n", port_);
    }
    if (reactor->register_handler(&rtcp_handler_, ACE_Event_Handler::READ_MASK) < 0)
    {
	printf("Register_handler for rtcp_handler for port %d failed\n", port_);
    }

}

Reflector::~Reflector()
{
    printf("Reflector::~Reflector() port=%d\n", port_);
}


void Reflector::add_address(char *address)
{
    ACE_INET_Addr addr(port_, address);
    
#ifdef DEBUG
    printf("Reflector::add_address(%s) %s/%d\n",
	   address, addr.get_host_name(), addr.get_port_number());
#endif

    int which = -1;
    for (int i = 0; i < n_addresses_; i++)
    {
#ifdef DEBUG
	printf("Compare to %d: %s/%d\n", i,
	       rtp_addresses_[i]->get_host_name(),
	       rtp_addresses_[i]->get_port_number());
#endif
	if (*(rtp_addresses_[i]) == addr)
	{
#ifdef DEBUG
	    printf("already have it\n");
#endif
	    which = i;
	    break;
	}
    }
    if (which == -1)
    {
	rtp_addresses_[n_addresses_] = new ACE_INET_Addr(addr);
	rtcp_addresses_[n_addresses_] = new ACE_INET_Addr(port_ + 1, address);
	n_addresses_++;
#ifdef DEBUG
	printf("add address %d\n", i);
#endif
    }
}

void Reflector::add_address(ACE_INET_Addr &src)
{
    ACE_INET_Addr addr(port_, src.get_ip_address());
    
#ifdef DEBUG
    printf("Reflector::add_address %s/%d\n",
	   addr.get_host_name(), addr.get_port_number());
#endif

    int which = -1;
    for (int i = 0; i < n_addresses_; i++)
    {
#ifdef DEBUG
	printf("Compare to %d: %s/%d\n", i,
	       rtp_addresses_[i]->get_host_name(),
	       rtp_addresses_[i]->get_port_number());
#endif
	if (*(rtp_addresses_[i]) == addr)
	{
#ifdef DEBUG
	    printf("already have it\n");
#endif
	    which = i;
	    break;
	}
    }
    if (which == -1)
    {
	rtp_addresses_[n_addresses_] = new ACE_INET_Addr(addr);
	rtcp_addresses_[n_addresses_] = new ACE_INET_Addr(port_ + 1, addr.get_ip_address());
	n_addresses_++;
#ifdef DEBUG
	printf("add address %d\n", i);
#endif
    }
}

void Reflector::remove_address(char *address)
{
    ACE_INET_Addr addr(port_, address);
    
#ifdef DEBUG
    printf("Reflector::remove_address(%s) %s/%d\n",
	   address, addr.get_host_name(), addr.get_port_number());
#endif

    int which = -1;
    for (int i = 0; i < n_addresses_; i++)
    {
#ifdef DEBUG
	printf("Compare to %d: %s/%d\n", i,
	       rtp_addresses_[i]->get_host_name(),
	       rtp_addresses_[i]->get_port_number());
#endif
	if (*(rtp_addresses_[i]) == addr)
	{
	    which = i;
	    break;
	}
    }
    if (which == -1)
    {
#ifdef DEBUG
	printf("address not present\n");
#endif
    }
    else
    {
	delete rtp_addresses_[which];
	delete rtcp_addresses_[which];
	for (int i = which; i < n_addresses_ - 1; i++)
	{
	    rtp_addresses_[i] = rtp_addresses_[i+i];
	    rtcp_addresses_[i] = rtcp_addresses_[i+i];
	}
	n_addresses_--;
    }
}

void Reflector::handle_packet(RTP_Packet::RTP_Header *packet_header,
				char *packet, int packet_size,
				ACE_INET_Addr &packet_source)
{
    
#ifdef PKT_DEBUG
    printf("Reflector: got rtp packet of length %d\n", packet_size);
    printf("flags=%x v=%d pad=%d ext=%d cc=%d marker=%d pt=%d seq=%d ssrc=%x\n",
	   
	   packet_header->get_flags(),
	   packet_header->get_version(),
	   packet_header->get_padding(),
	   packet_header->get_extension(),
	   packet_header->get_csrc_count(),
	   packet_header->get_marker(),
	   packet_header->get_packet_type(),
	   packet_header->get_sequence(),
	   packet_header->get_ssrc());
#endif

#ifdef PKT_DEBUG
    printf("rtp source: %s/%d %x\n", packet_source.get_host_name(),
	   packet_source.get_port_number(), packet_source.get_ip_address());
#endif

    for (int i = 0; i < n_addresses_; i++)
    {
	if (packet_source.get_ip_address() == rtp_addresses_[i]->get_ip_address())
	{
#ifdef PKT_DEBUG
	    printf("Skipping send to %s/%d\n", packet_source.get_host_name(),
		   packet_source.get_port_number());
#endif
	}
	else
	{
#ifdef PKT_DEBUG
	    printf("Sending rtp addr=%x len=%d to %s/%d\n",
		   packet, packet_size,
		   packet_source.get_host_name(),
		   packet_source.get_port_number());
#endif
	    send_socket_.send(packet, packet_size, *(rtp_addresses_[i]));
	}
    }
}
    
void Reflector::handle_packet(RTCP_Packet::RTCP_Header *packet_header,
				char *packet, int packet_size,
				ACE_INET_Addr &packet_source)
{

#ifdef DEBUG
    printf("Reflector: got rtcp packet of length %d\n", packet_size);
    printf("flags=%x version=%d padding=%d rc=%d pt=%d len=%d ssrc=%x\n",
	   packet_header->get_flags(),
	   packet_header->get_version(),
	   packet_header->get_padding(),
	   packet_header->get_reception_count(),
	   packet_header->get_packet_type(),
	   packet_header->get_length(),
	   packet_header->get_ssrc());
#endif

#ifdef DEBUG
    printf("rtcp source: %s/%d %x\n", packet_source.get_host_name(),
	   packet_source.get_port_number(), packet_source.get_ip_address());
#endif

    int found = 0;
    for (int i = 0; i < n_addresses_; i++)
    {
	if (packet_source.get_ip_address() == rtcp_addresses_[i]->get_ip_address())
	{
	    found = 1;
#ifdef DEBUG
	    printf("Skipping send to %s/%d\n", packet_source.get_host_name(),
		   packet_source.get_port_number());
#endif
	}
	else
	{
#ifdef DEBUG
	    printf("Sending rtcp addr=%x len=%d to %s/%d\n",
		   packet, packet_size,
		   rtcp_addresses_[i]->get_host_name(),
		   rtcp_addresses_[i]->get_port_number());
#endif

	    send_socket_.send(packet, packet_size, *(rtcp_addresses_[i]));
	}
    }

    /*
     * If we're getting RTCP from a host who we aren't sending to
     * currently, add the host to the list. Mark this host as
     * requring a timeout as well.
     */
    if (!found)
    {
	printf("Adding host %s via rtcp\n", packet_source.get_host_name());
	add_address(packet_source);
    }

}

void Reflector::handle_subpacket(RTCP_Packet::RTCP_Header *packet_header,
				   char *packet, int packet_size,
				   ACE_INET_Addr &packet_source)
{

}

int Reflector::handle_timeout(const ACE_Time_Value &tv, const void *arg)
{
    
    return 0;
}
