#ifndef _reflector_h
#define _reflector_h

#define MAX_ADDRESSES 1024

#include "RTP_Packet.h"
#include "RTCP_Packet.h"
#include "Packet_Handler.h"
#include "RTP_Packet_Handler.h"
#include "RTCP_Packet_Handler.h"
#include "RTP_Packet_Sink.h"
#include "RTCP_Packet_Sink.h"

#include "ace/Event_Handler.h"

class ACE_Reactor;

class Reflector : public RTP_Packet_Sink,
		  public RTCP_Packet_Sink,
		  public ACE_Event_Handler
{
public:
    Reflector(unsigned short port, ACE_Reactor *reactor);
    ~Reflector();

    void add_address(char *address);
    void add_address(ACE_INET_Addr &addr);
    void remove_address(char *address);

    int matches_port(unsigned short port) { return port == port_; };
    
    virtual void handle_packet(RTP_Packet::RTP_Header *packet_header,
			       char *packet, int packet_size,
			       ACE_INET_Addr &packet_source);
    
    virtual void handle_packet(RTCP_Packet::RTCP_Header *packet_header,
			       char *packet, int packet_size,
			       ACE_INET_Addr &packet_source);

    virtual void handle_subpacket(RTCP_Packet::RTCP_Header *packet_header,
				  char *packet, int packet_size,
				  ACE_INET_Addr &packet_source);

    virtual int handle_timeout(const ACE_Time_Value &tv, const void *arg);

protected:

    unsigned short port_;
    ACE_Reactor *reactor_;
    ACE_INET_Addr *rtp_addresses_[MAX_ADDRESSES];
    ACE_INET_Addr *rtcp_addresses_[MAX_ADDRESSES];
    int n_addresses_;

    ACE_INET_Addr rtp_addr_, rtcp_addr_;
    ACE_SOCK_Dgram rtp_socket_, rtcp_socket_;
    RTP_Packet_Handler rtp_handler_;
    RTCP_Packet_Handler rtcp_handler_;

    ACE_SOCK_Dgram send_socket_;
    
    
};

#endif
