#include "SSRC_Hash.h"

#include "SSRC_Writer.h"

SSRC_Hash::SSRC_Hash(int size) : size_(size)
{
    elements_ = new SSRC_Hash_Bucket [size_];

    for (int i = 0; i < size_; i++)
    {
	elements_[i].next_ = 0;
    }
}

SSRC_Hash::~SSRC_Hash()
{
    delete elements_;
}

void SSRC_Hash::add(long ssrc, SSRC_Writer *dw)
{
    SSRC_Hash_Bucket *b, *nb;

    b = get_bucket(ssrc);
    nb = new SSRC_Hash_Bucket(ssrc, dw, b->next_);
    b->next_ = nb;
}

void SSRC_Hash::remove(long ssrc)
{
    SSRC_Hash_Bucket *b, *lb;

    lb = get_bucket(ssrc);
    b = lb->next_;
    
    while (b != NULL && b->ssrc_ != ssrc)
    {
	b = b->next_;
	lb = b;
    }

    if (b)
    {
	if (b->ssrc_ != ssrc)
	{
	    printf("aiee\n");
	    exit(1);
	}
	else
	{
	    lb->next_ = b->next_;
	}
    }
    else
    {
	printf("SSRC_hash::remove: didn't find ssrc %x\n", ssrc);
    }
}

SSRC_Writer *SSRC_Hash::lookup(long ssrc)
{
    SSRC_Hash_Bucket *b;

    b = get_bucket(ssrc);

    b = b->next_;
    while (b)
    {
	if (b->ssrc_ == ssrc)
	    return b->dw_;
	b = b->next_;
    }
    return 0;
}



SSRC_Hash_Bucket *SSRC_Hash::get_bucket(long ssrc)
{
    unsigned int hash;

    hash = (unsigned int) ssrc % size_;
    return &(elements_[hash]);
}

void SSRC_Hash::call_tick(ACE_Time_Value tv, ACE_Time_Value last_tv, ACE_Time_Value delta)
{
    SSRC_Hash_Bucket *b;
    int i;

    for (i = 0; i < size_; i++)
    {
	for (b = elements_[i].next_; b != NULL; b = b->next_)
	{
	    if (b->dw_ != NULL)
	    {
	        b->dw_->tick(tv, last_tv, delta);
	    }
	}
    }
}

void SSRC_Hash::call_terminate()
{
    SSRC_Hash_Bucket *b;
    int i;

    for (i = 0; i < size_; i++)
    {
	for (b = elements_[i].next_; b != NULL; b = b->next_)
	{
	    if (b->dw_ != NULL)
	    {
	        b->dw_->terminate();
	    }
	}
    }
}

SSRC_Hash_Bucket::SSRC_Hash_Bucket(long ssrc, SSRC_Writer *dw,
				   SSRC_Hash_Bucket *next) :
				   ssrc_(ssrc),
				   dw_(dw),
				   next_(next)
{
}

SSRC_Hash_Bucket::SSRC_Hash_Bucket() :
				   ssrc_(0L),
				   dw_(0),
				   next_(0)
{
}

