#ifndef _SSRC_Reader_h
#define _SSRC_Reader_h

#include "ace/Event_Handler.h"
#include "ace/INET_Addr.h"
#include "RTP_Dump.h"

#include "defs.h"

class ACE_Reactor;
class Writer;
class Destination;

class SSRC_Reader : public ACE_Event_Handler
{
public:

    enum TimerType
    {
	Counter = 0,
	Update,
	RTCP_Report,
	MaxTimers
    };
    
    SSRC_Reader(char *path, Writer *writer, ACE_Reactor *reactor,
		int n_packet_types, int *packet_types);

    void select_destinations(int n_destinations, Destination **destinations);

    void start(ACE_Time_Value &playback_start, ACE_Time_Value &base_offset);
    void stop();
    void reset();

    char *path() { return path_; }
    ACE_Time_Value &start_time() { return start_time_; } ;

    double absolute_elapsed_time() {
	return ((double) packet_timestamp_ - absolute_starting_timestamp_) /
	    (double) timestamp_frequency_; };


    unsigned long ssrc() { return ssrc_; } ;

    void set_leader(void);
    void set_leader(SSRC_Reader *leader);
    void stream_started(SSRC_Reader *s);
    void stream_finished(SSRC_Reader *s);
    
protected:
    char path_[1024];

    FILE *fp_;
    char *fp_buffer_;
    Writer *writer_;
    ACE_Reactor *reactor_;

    SSRC_Reader *leader_;
    int n_streams_started_;

    int am_leader(void) { return leader_ == (SSRC_Reader *) NULL; } ;

    unsigned long ssrc_;

    int n_packet_types_;
    int *packet_types_;

    Destination **destinations_;
    int n_destinations_;

    void open_file();
    void read_info_file();
    void handle_info_item(char *, char *);
    int read_header();
    int read_packet();
    void read_and_send();
    void send_packet();
    void done_sending();
    void send_counter_update();

    int timer_ids_[MaxTimers];
    int reader_done_;

    ACE_Time_Value start_time_;
    ACE_Time_Value mapped_start_;
    ACE_INET_Addr original_source_;

    RTP_Dump_Header header_;
    RTP_Dump_Packet packet_;
    ACE_Time_Value packet_offset_;
    double playback_start_d_;
    int ptype_;
    unsigned long packet_timestamp_;
    unsigned long last_packet_timestamp_;
    unsigned long starting_timestamp_;
    unsigned long absolute_starting_timestamp_;
    unsigned long timestamp_frequency_;
    unsigned long synch_rtp_timestamp_;
    double synch_ntp_timestamp_;
    int found_rtp_packet_;

    // Stuff for RTCP reports

    ACE_Time_Value rtcp_interval_;

    double Nsynch_, Nstart_;
    double Rsynch_, Rstart_;
    double F_;

    double rtp_to_ntp(unsigned long);
    double rtp_to_rel_ntp(unsigned long);

    unsigned long C1_, C2_;
    ACE_Time_Value C3_;

    char packet_buffer_[MAX_PACKET];
    int packet_buffer_length_;

    double compute_playout_delay();

    virtual int handle_timeout(const ACE_Time_Value &tv, const void *arg);


    
};

#endif /* _SSRC_Reader_h */
