#ifndef _SSRC_Writer_h
#define _SSRC_Writer_h

class Disk_Writer;

#include "RTP_Packet.h"
#include "RTCP_Packet.h"
#include "RTP_Dump.h"

#include "ace/INET_Addr.h"
#include "ace/Time_Value.h"

#ifdef USE_IBM_CL
#include <ikeyset.h>
#endif

/*
 * Write a file for this data stream.
 *
 * Don't actually start writing until an rtp packet comes in.
 *
 * While we're not writing, save the last RTCP packet seen, so that
 * we can start the saved file with a RTCP packet.
 *
 */

class SSRC_Writer
{
public:

    SSRC_Writer(Disk_Writer *DW, long ssrc);
    ~SSRC_Writer();

#ifdef USE_IBM_CL
    const long & const ssrc() { return ssrc_; } ;
#else
    long ssrc() { return ssrc_; } ;
#endif

    virtual void handle_packet(RTP_Packet::RTP_Header *packet_header,
			       char *packet, int packet_size,
			       ACE_INET_Addr &packet_source,
			       int initial_packet = 0);
    
    virtual void handle_packet(RTCP_Packet::RTCP_Header *packet_header,
			       char *packet, int packet_size,
			       ACE_INET_Addr &packet_source,
			       int initial_packet = 0);

    virtual void handle_subpacket(RTCP_Packet::RTCP_Header *packet_header,
				  char *packet, int packet_size,
				  ACE_INET_Addr &packet_source);

    void terminate();

    void tick(const ACE_Time_Value &this_tick,
	      const ACE_Time_Value &last_tick,
	      const ACE_Time_Value &delta);

    
protected:

    void open_file();
    void write_packet(char *packet, int length, int type);
    void write_file_header(ACE_INET_Addr &source);
    void write_initial_info();
    void write_final_info();

    Disk_Writer *disk_writer_;
    char filename_[MAXPATHLEN];
    FILE *fp_;
    char *fp_buffer_;
    ACE_Time_Value start_time_;
    unsigned long start_ms_;
    long ssrc_;
    int last_rtp_seq;
    int bytes_;
    int missed_packets_;
    int packets_seen_;

    int have_synch_info_;

    char info_filename_[MAXPATHLEN];
    FILE *info_fp_;

    /*
     * Storage for the don't-write-the-file-header-until-data-comes-in
     * mechanism.
     */
    int writing_output_;
    RTCP_Packet saved_rtcp_packet_;
    int saved_rtcp_packet_size_;
    int have_saved_rtcp_packet_;

    /*
     * Map of RTP packet types that we've seen in this data stream.
     */
    char types_seen_[128];

    /*
     * Put this in here so we don't have to keep allocating stack space
     * for it.
     */
    RTP_Dump_Packet rd_packet_;
};


#ifdef USE_IBM_CL
long const & key(SSRC_Writer * const s);

typedef IKeySet<SSRC_Writer *, long> SSRC_Writer_KeySet;
typedef IKeySet<SSRC_Writer *, long>::Cursor SSRC_Writer_Cursor;
#endif

#endif /* _SSRC_Writer_h */

/*
$Log: SSRC_Writer.h,v $
Revision 1.7  1996/02/29 16:36:43  olson
Imported port to latest ACE 4.0 beta; largely a variable renaming.

This version also requires the _info file for media streams to have a ntp/rtp
synch field. This should actually go away and be replaced by having the
reader read the SR packets in the data stream.

Revision 1.5  1996/01/26 16:49:12  olson
Lotsa new stuff.

Revision 1.4  1995/10/22 23:47:18  olson
Added support for initial info file writing

*/
