#ifndef _Writer_h
#define _Writer_h

/*
 * This class manages the overall write-streams-to-the-net functionality.
 *
 * Again, it's an ACE_Event_Handler so it can catch signals and timer events.
 */

#include "nexus.h"
#include "ace/Event_Handler.h"
#include "RTCP_Packet_Sink.h"

class ACE_Reactor;
class SSRC_Reader;
class Destination;
class RTCP_Packet_Handler;

#define MAX_READERS 1024
#define MAX_DESTINATIONS 1024
#define MAX_CLIENTS 1024

/*
 * EW EW EW
 */

#if defined (ACE_HAS_SIGINFO_T)
#define HANDLE_SIGNAL_ARGS , siginfo_t * = 0, ucontext_t * = 0
#else
#define HANDLE_SIGNAL_ARGS
#endif /* ACE_HAS_SIGINFO_T */

class Writer : public ACE_Event_Handler, public RTCP_Packet_Sink
{
public:
    Writer(ACE_Reactor *reactor);

    static void add_client_handler(void *address, nexus_buffer_t *buffer);
    static void add_file_handler(void *address, nexus_buffer_t *buffer);
    static void remove_file_handler(void *address, nexus_buffer_t *buffer);
    static void add_destination_handler(void *address, nexus_buffer_t *buffer);
    static void remove_destination_handler(void *address,
					   nexus_buffer_t *buffer);
    static void start_handler(void *address, nexus_buffer_t *buffer);
    static void stop_handler(void *address, nexus_buffer_t *buffer);
    static void reset_handler(void *address, nexus_buffer_t *buffer);
    static void quit_handler(void *address, nexus_buffer_t *buffer);

    void add_client(nexus_global_pointer_t *client_gp);
    void add_file(char *path, int n_packet_types, int *packet_types);
    void remove_file(char *path);
    void add_destination(char *host, int port,
			 nexus_global_pointer_t *cb_gp,
			 int n_packet_types, int *packet_types);
    void remove_destination(char *host, int port);
    void start();
    void stop();
    void reset();
    void quit();

    void send_rtp_packet(int rtp_type, char *packet, int len);
    void send_rtcp_packet(char *packet, int len);
    void send_rtcp_packet(int rtp_type, char *packet, int len);

    int finished() { return finished_; } ;
    ACE_Time_Value &playback_start() { return playback_start_; };

    void reader_finished(SSRC_Reader *s);
    void readers_finished(void);

protected:
    ACE_Reactor *reactor_;
    int finished_;
    int running_;

    int n_listening_ports_;
    int listening_ports_[1024];
    RTCP_Packet_Handler *listening_handlers_[1024];

    int n_active_clients_;
    long active_clients_[1024];	// the ssrc of the client
    int n_finished_clients_;
    
    nexus_global_pointer_t *clients_[MAX_CLIENTS];
    int n_clients_;

    int counter_id_;

    ACE_Time_Value base_offset_;
    ACE_Time_Value playback_start_;

    SSRC_Reader *readers_[MAX_READERS];
    int n_readers_;

    int n_readers_running_;

    // The leader is the reader with the minimum start time.
    // It provides timestamps to the destinations.
    SSRC_Reader *leader_;

    Destination *destinations_[MAX_DESTINATIONS];
    int n_destinations_;
    int n_destinations_done_;

    void add_reader(SSRC_Reader *reader);
    void remove_reader(SSRC_Reader *reader);

    void add_destination(Destination *destination);
    void remove_destination(Destination *destination);
    Destination *find_destination(char *host, int port);

    virtual int handle_timeout(const ACE_Time_Value &tv, const void *arg);
    virtual int handle_signal (ACE_HANDLE signum HANDLE_SIGNAL_ARGS);

    void send_counter_update(void);

    virtual void handle_packet(RTCP_Packet::RTCP_Header *packet_header,
			       char *packet, int packet_size,
			       ACE_INET_Addr &packet_source);

    virtual void handle_subpacket(RTCP_Packet::RTCP_Header *packet_header,
			       char *packet, int packet_size,
			       ACE_INET_Addr &packet_source) ;

    void ensure_listener(int port);
    
};


#endif /* _Writer_h */
