
/*
 * Read RTP packets. Arguments: list of ports to listen on
 * (each port must be an even number, per RTP spec).
 */

#include "Packet_Handler.h"
#include "Media_Stream.h"
#include "Disk_Writer.h"

#include "ace/Reactor.h"
#include "ace/Get_Opt.h"

#include <stdio.h>
#include <sys/stat.h>

static void usage(void);

ACE_Reactor *glb_reactor;

void main(int argc, char **argv)
{
    int *ports, n_ports;
    Media_Stream **streams;
    ACE_Reactor *reactor;
    int i;
    char *output_directory;
    char *str;
    char *refl_host = NULL;
    struct stat s;
    ACE_Get_Opt get_opt(argc, argv, "o:r:");
    int c;

    output_directory = (char *) NULL;
    while ((c = get_opt()) != -1)
    {
	switch (c)
	{
	case 'o':
	    output_directory = new char[strlen(get_opt.optarg) + 1];
	    strcpy(output_directory, get_opt.optarg);
	    break;

#if 0
	case 'r':
	    refl_host = new char[strlen(get_opt.optarg) + 1];
	    strcpy(refl_host, get_opt.optarg);
	    printf("reflect to %s\n", refl_host);
	    break;
#endif

	default:
	    usage();
	    break;
	}
    }

    if (get_opt.optind >= argc)
	usage();

    if (output_directory)
    {
	if (stat(output_directory, &s) < 0)
	{
	    printf("Directory %s not found\n", output_directory);
	    exit(1);
	}

	if (!(S_ISDIR(s.st_mode)))
	{
	    printf("%s is not a directory\n", output_directory);
	    exit(1);
	}
	printf("Writing output to %s\n", output_directory);
    }
    else
    {
	printf("Not writing output\n");
    }
    
    n_ports = argc - get_opt.optind;
    printf("optind=%d n_ports=%d\n", get_opt.optind, n_ports);

    ports = new int[n_ports];
    streams = new Media_Stream *[n_ports];

    reactor = new ACE_Reactor;
    glb_reactor = reactor;
    Disk_Writer *disk = new Disk_Writer(output_directory);

    reactor->schedule_timer(disk, 0,
			    ACE_Time_Value(5, 0),
			    ACE_Time_Value(5, 0));

    reactor->register_handler(SIGHUP, disk);
    reactor->register_handler(SIGINT, disk);
    reactor->register_handler(SIGTERM, disk);

    for (i = 0; i < n_ports; i++)
    {
	ports[i] = atoi(argv[i + get_opt.optind]);
	if ((ports[i] % 2) != 0)
	{
	    fprintf(stderr, "%s: ports must be even numbers\n", argv[0]);
	    exit(1);
	}
	streams[i] = new Media_Stream(ports[i], disk, disk);
	streams[i]->register_with_reactor(reactor);
    }
    printf("hey, we're started\n");

//    disk->setup_reflector(refl_host, ports, n_ports);

    while (!disk->finished())
    {
	reactor->handle_events();
    }
    printf("Finished\n");
    disk->terminate();
}

static void usage(void)
{
    fprintf(stderr, "Usage: preader directory port-list\n");
    exit(1);
}
