#include "ace/Reactor.h"
#include "ace/Get_Opt.h"

#include "nexus.h"

#include "Nexus_Handler.h"
#include "Writer.h"

#include <stdio.h>

static void usage(void);

static void send_hello(nexus_global_pointer_t *gp, nexus_global_pointer_t *);

void register_handlers(void);

/*
 * Write an archived session back out to the net.
 *
 * This is a Nexus Lite application, so that the user can control
 * (stop, start, reset) it easily.
 *
 * Usage:
 *
 *    pwriter url
 *
 * Pwriter attaches to the url, and invokes the pwriter_init
 * rsr, passing a global pointer to itself.
 *
 * Pwriter responds to the following rsrs:
 *
 *    add_file 		Adds the given file to the list of streams to play
 *    remove_file	Removes the file
 *    add_destination   Adds the given host/port pair to the dest list
 *    remove_destination  Removes that destination
 *    start		Start playing
 *    stop		Stop playing
 *    reset		Stop playing, reset all files to their start
 */

static int pkg_init(int *argc, char ***argv)
{
    return 0;
}

static void pkg_usage()
{
}

static int pkg_params(char *buf, int size)
{
    return 0;
}

int NexusBoot()
{
    return 0;
}

void main(int argc, char **argv)
{
    nexus_node_t *nodes;
    int n_nodes;
    int c;
    char *url = NULL;
    nexus_global_pointer_t attach_gp;

    setbuf(stdout, 0);
    setbuf(stderr, 0);
    
    nexus_init(&argc, &argv, "PWRITER_ARGS", "nx",
	       pkg_init, pkg_usage, pkg_params, NULL, &nodes, &n_nodes);
    nexus_start();

    ACE_Get_Opt get_opt(argc, argv, "u:");

    register_handlers();

    printf("nexus started n_nodes=%d\n", n_nodes);

    while ((c = get_opt()) != -1)
    {
	switch (c)
	{
	case 'u':
	    url = new char[strlen(get_opt.optarg) + 1];
	    strcpy(url, get_opt.optarg);
	    break;

	default:
	    printf("Usage: %s [-u url] [files]\n", argv[0]);
	    exit(1);
	}
    }

    if (url)
	printf("have url=%s\n", url);
    else
	printf("No url\n");

    if (url)
    {

	/*
	 * Attach to the controlling process.
	 */

	printf("Attaching to %s\n", url);
    
	int rc = nexus_attach(url, &attach_gp);
	if (rc != 0)
	{
	    printf("attach to %s failed with rc=%d\n", url, rc);
	    exit(1);
	}
	
	printf("Attached to %s\n", url);
    }

    /*
     * We're attached, go ahead and do the ACE_Reactor setup.
     */

    ACE_Reactor *reactor = new ACE_Reactor;

    Writer *writer = new Writer(reactor);

    reactor->register_handler(SIGHUP, writer);
    reactor->register_handler(SIGINT, writer);
    reactor->register_handler(SIGTERM, writer);

    if (url)
    {
	nexus_global_pointer_t writer_gp;

	nexus_global_pointer(&writer_gp, (void *) writer);
	send_hello(&attach_gp, &writer_gp);
	nexus_destroy_global_pointer(&writer_gp);
    }
    else
    {
	/* Add the files, start playing */

	int i;
	int n_files;

	n_files = argc - get_opt.optind;

	for (i = 0; i < n_files; i++)
	{
	    char *file = argv[i + get_opt.optind];
	}
    }
    Nexus_Handler::register_with_nexus(reactor);

    printf("Handling events\n");
    
    while (!writer->finished())
    {
	reactor->handle_events();
    }

    nexus_destroy_global_pointer(&attach_gp);
    nexus_destroy_current_context(0);
}

static void send_hello(nexus_global_pointer_t *gp,
		       nexus_global_pointer_t *me_gp)
{
    nexus_buffer_t buf;
    int size, n_elts, e;
    
    nexus_init_remote_service_request(&buf, gp, "pwriter_init",
				      nexus_handler_hash("pwriter_init"));
    size = n_elts = 0;
    size += nexus_sizeof_global_pointer(&buf, me_gp, 1, &e);
    n_elts += e;
    nexus_set_buffer_size(&buf, size, n_elts);
    nexus_put_global_pointer(&buf, me_gp, 1);
    printf("Sending hello\n");
    nexus_send_remote_service_request(&buf);
    printf("Sent hello\n");
}

static void usage(void)
{
    fprintf(stderr, "Usage: pwriter url\n");
    exit(1);
}
