
#include "ace/SOCK_Dgram.h"
#include "ace/INET_Addr.h"

#if defined(ACE_HAS_IP_MULTICAST)
#include "ace/SOCK_Dgram_Mcast.h"

typedef ACE_SOCK_Dgram_Mcast SockType;

#else /* ACE_HAS_IP_MULTICAST */

typedef ACE_SOCK_Dgram SockType;

#endif /* ACE_HAS_IP_MULTICAST */

#include "ace/Reactor.h"

class Packet_Handler : public ACE_Event_Handler
{
public:
  Packet_Handler (u_short port, 
		 const char *mcast_addr, 
		 long time_interval = 0);
  ~Packet_Handler (void);

  virtual int handle_input (ACE_HANDLE fd);
  virtual ACE_HANDLE get_handle (void) const;

  ACE_Time_Value *wait_time (void);

private:
  char *message_;

  char buf_[4*BUFSIZ];

  int interval_;
  // time interval to log messages

  ACE_Time_Value *how_long_;
  ACE_Reactor    *reactor_;
  SockType   mcast_dgram_;
  
  ACE_INET_Addr  remote_addr_;
  ACE_INET_Addr  mcast_addr_;

};

ACE_HANDLE
Packet_Handler::get_handle (void) const
{
  return this->mcast_dgram_.get_handle ();
}

ACE_Time_Value *
Packet_Handler::wait_time (void)
{
  return this->how_long_;
}

Packet_Handler::Packet_Handler (u_short port, 
				const char *mcast_addr,
				long time_interval)
  : interval_ (time_interval),
#ifdef ACE_HAS_IP_MULTICAST
    mcast_addr_ (port, mcast_addr)
#else
    mcast_addr_ (port)
#endif
{

  printf("%x addr is %s %s %d (took in %d)\n", this,
	 mcast_addr_.get_host_name(),
	 mcast_addr_.get_host_addr(),
	 mcast_addr_.get_port_number(),
	 port);


  // use ACE_SOCK_Dgram_Mcast factory to subscribe to multicast group.

#ifdef ACE_HAS_IP_MULTICAST
  if (this->mcast_dgram_.subscribe (this->mcast_addr_) == -1)
    perror("can't subscribe to multicast group"), exit(1);
#else
  if (this->mcast_dgram_.open (this->mcast_addr_) == -1)
    perror("can't subscribe to multicast group"), exit(1);
  
#endif

  // Point to NULL so that we block in the beginning.
  this->how_long_ = 0;

  this->message_    = buf_;
}

// a destructor which emacs refuses to color blue
Packet_Handler::~Packet_Handler ()
{
#ifdef ACE_HAS_IP_MULTICAST
  this->mcast_dgram_.unsubscribe ();
#endif

}

int
Packet_Handler::handle_input (ACE_HANDLE fd)
{

    // receive message from multicast group
    int retcode = this->mcast_dgram_.recv (this->buf_, 
					   sizeof this->buf_, 
					   this->remote_addr_);
    if (retcode != -1)
    {
	printf("%x Read %d bytes\n", this, retcode);
	return 0;
    }
    else
	return -1;
};

int 
main(int argc, char *argv[])
{
    int duration = 5;

    // Instantiate a server which will receive messages for 5 seconds.
    Packet_Handler rtp_handler (9998, "224.2.5.6", duration);
    Packet_Handler rtcp_handler (9999, "224.2.5.6", duration);

    // Instance of the ACE_Reactor.
    ACE_Reactor reactor;

    if (reactor.register_handler (&rtp_handler,
				  ACE_Event_Handler::READ_MASK) == -1)
    {
//	LM_ERROR ((LOG_ERROR, "%p\n%a", "register_handler", 1));
	printf("register_handler failed: %s\n", strerror(errno));
    }
    
    if (reactor.register_handler (&rtcp_handler,
				  ACE_Event_Handler::READ_MASK) == -1)
    {
//	LM_ERROR ((LOG_ERROR, "%p\n%a", "register_handler", 1));
	printf("register_handler failed: %s\n", strerror(errno));
    }
    
    for (;;)
	reactor.handle_events ();

    cout << "\nserver done\n";
    return 0;
}
