
#include "Reflector.h"
#include "Nexus_Handler.h"
#include <ace/Get_Opt.h>
#include <ace/Reactor.h>

#include "nexuslite.h"

/*
 * Globals Ew
 */

static int n_ports, *ports;
static Reflector **reflectors;
static ACE_Reactor *reactor;

void register_handlers(void);

static int pkg_init(int *argc, char ***argv)
{
    return 0;
}

static void pkg_usage()
{
}

static int pkg_params(char *buf, int size)
{
    return 0;
}


static int data;
static int approve(void *user_arg, char *url, nexus_global_pointer_t *gp)
{
    nexus_global_pointer(gp, (void *) &data);
    printf("approving %s\n", url);
    return 0;
}

static void setup_nexus_attachment(unsigned short port)
{
    char *host;
    int rc;

    rc = nexus_allow_attach(&port, &host, approve, NULL);
    if (rc != 0)
    {
	printf("allow attach failed with rc=%d\n", rc);
	exit(1);
    }

    printf("Attachment URL: x-nexus://%s:%d/\n", host, port);
}

/*
 * Protocol: port addr-len addr
 */
static void add_address_handler(void *gpaddr, nexus_buffer_t *buf)
{
    int port, len, which;
    char *addr;

    nexus_get_int(buf, &port, 1);
    nexus_get_int(buf, &len, 1);
    addr = new char[len+1];
    nexus_get_char(buf, addr, len);
    addr[len] = '\0';

    which = -1;
    for (int i = 0; i < n_ports; i++)
    {
	if (reflectors[i]->matches_port(port))
	{
	    which = i;
	    break;
	}
    }

    if (which != -1)
    {
	reflectors[which]->add_address(addr);
    }
    else
    {
	printf("add_address_handler(): port %d not found\n", port);
    }
}

static void remove_address_handler(void *gpaddr, nexus_buffer_t *buf)
{
    int port, len, which;
    char *addr;

    nexus_get_int(buf, &port, 1);
    nexus_get_int(buf, &len, 1);
    addr = new char[len+1];
    nexus_get_char(buf, addr, len);
    addr[len] = '\0';

    which = -1;
    for (int i = 0; i < n_ports; i++)
    {
	if (reflectors[i]->matches_port(port))
	{
	    which = i;
	    break;
	}
    }

    if (which != -1)
    {
	reflectors[which]->remove_address(addr);
    }
    else
    {
	printf("remove_address_handler(): port %d not found\n", port);
    }
}


static nexus_handler_t handlers[] =
{ \
  {"add_address", 113, NEXUS_HANDLER_TYPE_NON_THREADED,
       (nexus_handler_func_t) add_address_handler},
  {"remove_address", 470, NEXUS_HANDLER_TYPE_NON_THREADED,
       (nexus_handler_func_t) remove_address_handler},
  {(char *) NULL,
       0,
       NEXUS_HANDLER_TYPE_NON_THREADED,
       (nexus_handler_func_t) NULL},
};

int NexusBoot()
{
    nexus_register_handlers(handlers);
	
    return 0;
}

static void usage()
{
    printf("bonehead\n");
}

void main(int argc, char **argv)
{
    int c;
    int n_nodes;
    nexus_node_t *nodes;
    int port;

    nexus_init(&argc, &argv, "REFLECTOR_ARGS", "nx",
	       pkg_init, pkg_usage, pkg_params, NULL, &nodes, &n_nodes);
    nexus_start();

    port = 29159;

    ACE_Get_Opt get_opt(argc, argv, "p:");
    while ((c = get_opt()) != -1)
    {
	switch (c)
	{
	case 'p':
	    port = atoi(get_opt.optarg);
	    break;

	default:
	    usage();
	    break;
	}
     }

    if (get_opt.optind >= argc)
	usage();

    setup_nexus_attachment(port);

    n_ports = argc - get_opt.optind;
    printf("optind=%d n_ports=%d\n", get_opt.optind, n_ports);

    ports = new int[n_ports];
    reflectors = new Reflector *[n_ports];

    reactor = new ACE_Reactor;

    Nexus_Handler::register_with_nexus(reactor);

    for (int i = 0; i < n_ports; i++)
    {
	ports[i] = atoi(argv[i + get_opt.optind]);
	printf("starting reflector on port %d\n", ports[i]);
	reflectors[i] = new Reflector(ports[i], reactor);
    }

    while (1)
    {
	reactor->handle_events();
    }

}
